from fastapi import HTTPException
from kiss_ai_stack_server.events.event_handlers import on_auth
from kiss_ai_stack_server.models.db.session import Session
from kiss_ai_stack_server.services.session_service import SessionService
from kiss_ai_stack_server.services.token_service import TokenService
from kiss_ai_stack_types.enums import SessionScope
from kiss_ai_stack_types.models import SessionRequest, SessionResponse
from starlette.status import HTTP_401_UNAUTHORIZED, HTTP_400_BAD_REQUEST


@on_auth
async def handle_auth(data: SessionRequest, session: Session = None) -> SessionResponse:
    scope = SessionScope(data.scope) if data.scope else None
    client_id = data.client_id
    client_secret = data.client_secret

    if (client_id and client_secret) or scope:
        session = await SessionService.create_or_get_session(
            scope=scope,
            client_id=client_id,
            client_secret=client_secret
        )
        if not session:
            raise HTTPException(
                status_code=HTTP_400_BAD_REQUEST,
                detail='Invalid session request.'
            )
        access_token = TokenService.generate_token(client_id=session.client_id, scope=session.scope)

        return SessionResponse(
            client_id=session.client_id,
            client_secret=session.client_secret,
            access_token=access_token
        )
    else:
        raise HTTPException(
            status_code=HTTP_401_UNAUTHORIZED,
            detail='Value error, Either `scope` or both `client_id` and `client_secret` must be provided.'
        )



import asyncio
from typing import Optional

import uvicorn
from fastapi import FastAPI

from kiss_ai_stack_server.routers.rest_router import rest_router
from kiss_ai_stack_server.routers.ws_router import ws_router
from kiss_ai_stack_server.services.session_service import SessionService
from kiss_ai_stack_server.utilities.session_auth_middleware import SessionAuthMiddleware


async def bootstrap_session_schema():
    """
    Initialize database connection and build schemas before starting the application
    """
    await SessionService.connect()
    await SessionService.build_schema()


def get_agent_server_app():
    app = FastAPI(
        title='KISS AI Stack Server',
        description='Enterprise-grade server for the KISS AI Stack',
        version='0.1.0-alpha'
    )

    # Add middleware for session authentication
    app.add_middleware(SessionAuthMiddleware)

    app.include_router(rest_router)
    app.include_router(ws_router)

    return app


def agent_server(config: Optional[uvicorn.Config] = None) -> uvicorn.Server:
    """
    Prepare the Uvicorn server instance for execution

    :param config: uvicorn Server configurations.
    :returns: uvicorn.Server object.
    """
    # Default configuration for Uvicorn server if not provided
    if config is None:
        config = uvicorn.Config(
            app=get_agent_server_app(),
            host='0.0.0.0',
            port=8080,
            reload=True,
            log_level='info'
        )

    return uvicorn.Server(config)

async def main():
    await bootstrap_session_schema()
    await agent_server().serve()

asyncio.run(main())



from fastapi import HTTPException
from kiss_ai_stack_types.enums import ServerEvent
from kiss_ai_stack_types.models import QueryRequestBody, GenericResponseBody, SessionResponse, DocumentsRequestBody

from kiss_ai_stack_server.events import event_handlers
from kiss_ai_stack_server.models.db import Session


def handle_server_event(
        event: ServerEvent,
        data: QueryRequestBody | DocumentsRequestBody,
        session: Session = None
) -> GenericResponseBody | SessionResponse:
    """
    Generic handler for server events with consistent error handling.

    :param event: (ServerEvent) The specific server event to handle
    :param data: Inputs to pass into the event
    :param session: Authorized client data
    :returns: The result of the event handler
    :raises HTTPException: If no handler is found for the event
    """
    if event in event_handlers:
        return event_handlers[event](data, session)
    raise HTTPException(
        status_code=404,
        detail=f'{event} handler not found'
    )



from typing import Callable, Dict

from kiss_ai_stack_types.enums import ServerEvent

event_handlers: Dict[str, Callable] = {}


def on_auth(func: Callable):
    event_handlers[ServerEvent.ON_AUTH] = func
    return func


def on_close(func: Callable):
    event_handlers[ServerEvent.ON_CLOSE] = func
    return func


def on_init(func: Callable):
    event_handlers[ServerEvent.ON_INIT] = func
    return func


def on_query(func: Callable):
    event_handlers[ServerEvent.ON_QUERY] = func
    return func


def on_store(func: Callable):
    event_handlers[ServerEvent.ON_STORE] = func
    return func



from kiss_ai_stack_server.events.event_handlers import on_init
from kiss_ai_stack_server.models.db.session import Session
from kiss_ai_stack_server.services.kiss_ai_agent_service import KissAIAgentService
from kiss_ai_stack_types.enums import SessionScope
from kiss_ai_stack_types.models import QueryRequestBody, GenericResponseBody


@on_init
async def handle_init(data: QueryRequestBody, session: Session) -> GenericResponseBody:
    stack = KissAIAgentService()
    temporary = session.scope == SessionScope.TEMPORARY
    await stack.bootstrap_agent(agent_id=session.client_id, temporary=temporary)

    return GenericResponseBody(
        agent_id=session.client_id,
        result='Greetings!'
    )



import secrets
import uuid

from kiss_ai_stack_types.enums import SessionScope
from tortoise import fields, models


class Session(models.Model):
    id = fields.IntField(pk=True)
    client_id = fields.CharField(max_length=255, unique=True, null=False, default=lambda: str(uuid.uuid4()))
    client_secret = fields.CharField(max_length=255, null=False, default=lambda: secrets.token_urlsafe(32))
    scope = fields.CharEnumField(SessionScope)
    created_at = fields.DatetimeField(auto_now_add=True)
    updated_at = fields.DatetimeField(auto_now=True)
    status = fields.BooleanField(default=True)

    class Meta:
        table = 'sessions'

    def verify_secret(self, secret: str) -> bool:
        """Verify the client secret"""
        return self.client_secret == secret



from fastapi import APIRouter, HTTPException
from kiss_ai_stack_types.enums import ServerEvent
from kiss_ai_stack_types.models import SessionRequest, SessionResponse, QueryRequestBody, DocumentsRequestBody
from starlette.requests import Request

from kiss_ai_stack_server.utilities.server_event_utils import handle_server_event

rest_router = APIRouter()


@rest_router.post('/auth')
async def exec_auth(data: SessionRequest) -> SessionResponse:
    return await handle_server_event(ServerEvent.ON_AUTH, data)


@rest_router.post('/sessions')
async def exec_session_action(action: str, data: QueryRequestBody, request: Request):
    """
    Perform an action on the session based on the query parameter 'action'.

    Supported actions:
    - close: Close the session
    - init: Initialize the session
    """
    event_map = {
        'close': ServerEvent.ON_CLOSE,
        'init': ServerEvent.ON_INIT
    }

    if action not in event_map:
        raise HTTPException(status_code=400, detail=f'Invalid action: {action}')

    session = getattr(request.state, 'session', None)
    return await handle_server_event(event_map[action], data, session)


@rest_router.post('/queries')
async def exec_query(data: QueryRequestBody, request: Request):
    session = getattr(request.state, 'session', None)
    return await handle_server_event(ServerEvent.ON_QUERY, data, session)


@rest_router.post('/documents')
async def exec_store(data: DocumentsRequestBody, request: Request):
    session = getattr(request.state, 'session', None)
    return await handle_server_event(ServerEvent.ON_STORE, data, session)



from fastapi import APIRouter, WebSocket, WebSocketDisconnect, HTTPException
from fastapi.websockets import WebSocketState
from kiss_ai_stack.core.utilities.logger import LOG
from kiss_ai_stack_types.enums import ServerEvent
from kiss_ai_stack_types.models import QueryRequestBody, DocumentsRequestBody

from kiss_ai_stack_server.services.session_service import SessionService
from kiss_ai_stack_server.services.token_service import TokenService
from kiss_ai_stack_server.utilities.server_event_utils import handle_server_event

ws_router = APIRouter()


@ws_router.websocket('/ws')
async def websocket_endpoint(websocket: WebSocket):
    """
    WebSocket endpoint with JWT authentication and session retrieval.

    :param websocket: (WebSocket) The active WebSocket connection
    """
    try:
        authorization = websocket.headers.get('Authorization')
        if not authorization or not authorization.startswith('Bearer '):
            await websocket.close(code=4001, reason='Missing or invalid Authorization header')
            return

        access_token = authorization[7:]
        try:
            payload = TokenService.decode_token(access_token)
        except Exception:
            await websocket.close(code=4001, reason="Invalid token")
            return
        session = await SessionService.get_active_session_by_client_id(
            client_id=payload['sub']
        )
        if not session:
            await websocket.close(code=4001, reason="Invalid or inactive session")
            return

    except Exception:
        await websocket.close(code=4001, reason="Authentication failed")
        return

    await websocket.accept()

    try:
        while websocket.client_state != WebSocketState.DISCONNECTED:
            try:
                data = await websocket.receive_json()
                event_str = data.get('event')

                try:
                    event = ServerEvent(event_str)
                except ValueError:
                    raise HTTPException(
                        status_code=400,
                        detail=f'Invalid event: {event_str}'
                    )
                match event:
                    case ServerEvent.ON_STORE:
                        request_data = DocumentsRequestBody(**data.get('data', {}))
                    case _:
                        request_data = QueryRequestBody(**data.get('data', {}))

                response = await handle_server_event(
                    event,
                    request_data,
                    session=session
                )
                await websocket.send_json({
                    'event': event_str,
                    'result': response.model_dump()
                })

            except HTTPException as http_exc:
                await websocket.send_json({
                    'error': True,
                    'status_code': http_exc.status_code,
                    'detail': http_exc.detail
                })
            except ValueError as ve:
                LOG.warning(f'Invalid data received: {ve}')
                await websocket.send_json({
                    'error': True,
                    'status_code': 400,
                    'detail': 'Invalid data'
                })

    except WebSocketDisconnect:
        LOG.warning('WebSocket disconnected')
    except Exception as e:
        LOG.error(f'Unexpected WebSocket error: {e}')
    finally:
        if websocket.client_state != WebSocketState.DISCONNECTED:
            await websocket.close()
        LOG.info('WebSocket connection closed')



import atexit
import os
import shutil
import tempfile
from functools import lru_cache


@lru_cache(maxsize=1)
def temp_file_manager():
    """
    Cached singleton factory function for AIAgentService.
    """
    return TemporaryFileManager()



class TemporaryFileManager:
    """
    Manages temporary files and directories with guaranteed cleanup
    """

    def __init__(self):
        self._temp_dirs = set()
        atexit.register(self.cleanup_all)

    def create_temp_dir(self) -> str:
        """
        Create a temporary directory and track it for cleanup

        :return: Path to the temporary directory
        """
        temp_dir = tempfile.mkdtemp()
        self._temp_dirs.add(temp_dir)
        return temp_dir

    def cleanup_dir(self, dir_path: str):
        """
        Remove a specific temporary directory

        :param dir_path: Path to the temporary directory
        """
        try:
            if dir_path in self._temp_dirs and os.path.exists(dir_path):
                shutil.rmtree(dir_path)
                self._temp_dirs.discard(dir_path)
        except Exception as e:
            print(f"Error cleaning up temp directory {dir_path}: {e}")

    def cleanup_all(self):
        """
        Remove all tracked temporary directories
        """
        for temp_dir in list(self._temp_dirs):
            self.cleanup_dir(temp_dir)

    def safe_file_path(self, temp_dir: str, filename: str) -> str:
        """
        Generate a safe file path within the temporary directory

        :param temp_dir: Base temporary directory
        :param filename: Original filename
        :return: Safe file path
        """
        safe_filename = os.path.basename(filename)
        return os.path.join(temp_dir, safe_filename)



from typing import List, Dict, Any, Optional

from kiss_ai_stack import AgentStack


class KissAIAgentService:
    _instance = None

    def __new__(cls):
        """
        Ensure only one instance of the class is created.

        :returns: The singleton instance of the class.
        """
        if cls._instance is None:
            cls._instance = super().__new__(cls)
            cls._instance.__stack = AgentStack()
        return cls._instance

    def __init__(self):
        pass

    async def bootstrap_agent(self, agent_id: str, temporary: Optional[bool] = True):
        """
        Initialize the AI agent stack.

        :param agent_id: Agent session's unique id, preferably client Id
        :param temporary: Whether to keep or cleanup stored docs
        """
        await self.__stack.bootstrap_agent(agent_id=agent_id, temporary=temporary)

    async def store_data(self, agent_id: str, files: List[str], metadata: Optional[Dict[str, Any]]):
        """
        Store documents using the AI agent stack core.

        :param agent_id: Agent's unique id, preferably client Id
        :param files: List of file paths to store
        :param metadata: Metadata for files/documents
        :returns: Status of storing documents
        """
        return await self.__stack.store_data(agent_id=agent_id, files=files, metadata=metadata)

    async def generate_answer(self, agent_id: str, query: any):
        """
        Process a query using the AI agent stack.

        :param agent_id: Agent session's unique id, preferably client Id
        :param query: User query/prompt input
        :returns: Answer for user prompt or query.
        """
        return await self.__stack.generate_answer(agent_id=agent_id, query=query)

    async def destroy_agent(self, agent_id: str, cleanup: bool = False):
        """
        Destroy the AI agent session and release resources.

        :param agent_id: Identifier of the agent to destroy.
        :param cleanup: Prompt to remove user data if RAG tools present, cleanup stored docs.
        """
        if hasattr(self.__stack.get_agent(agent_id=agent_id), 'destroy'):
            await self.__stack.destroy_agent(agent_id=agent_id, cleanup=cleanup)



import os
from datetime import datetime, timedelta, UTC
from typing import Optional, Dict

import jwt
from fastapi import HTTPException
from kiss_ai_stack_types.enums import SessionScope


class TokenService:
    SECRET_KEY = os.getenv('ACCESS_TOKEN_SECRET_KEY', 'your-secret-key')
    ALGORITHM = os.getenv('ACCESS_TOKEN_ALGORITHM', 'HS256')
    ACCESS_TOKEN_EXPIRE_MINUTES = int(os.getenv('ACCESS_TOKEN_EXPIRE_MINUTES', 30))

    @classmethod
    def generate_token(
            cls,
            client_id: str,
            scope: SessionScope,
            expires_delta: Optional[timedelta] = None
    ) -> str:
        payload = {
            'sub': client_id,
            'scope': scope.value,
            'exp': datetime.now(UTC) + (expires_delta or timedelta(minutes=cls.ACCESS_TOKEN_EXPIRE_MINUTES))
        }
        return jwt.encode(payload, cls.SECRET_KEY, algorithm=cls.ALGORITHM)

    @classmethod
    def decode_token(cls, token: str) -> Dict:
        """Decode and validate JWT token"""
        try:
            return jwt.decode(token, cls.SECRET_KEY, algorithms=[cls.ALGORITHM])
        except jwt.PyJWTError:
            raise HTTPException(status_code=401, detail="Invalid token")



from .event_handlers import *

from .on_auth import handle_auth
from .on_close import handle_close
from .on_init import handle_init
from .on_query import handle_query
from .on_store import handle_store



from fastapi import HTTPException, Request
from fastapi.responses import JSONResponse
from starlette.middleware.base import RequestResponseEndpoint, BaseHTTPMiddleware
from starlette.responses import Response
from starlette.status import HTTP_401_UNAUTHORIZED

from kiss_ai_stack_server.services.session_service import SessionService
from kiss_ai_stack_server.services.token_service import TokenService


class SessionAuthMiddleware(BaseHTTPMiddleware):
    async def dispatch(self, request: Request, call_next: RequestResponseEndpoint) -> Response:
        if request.scope['type'] == 'websocket':
            return await call_next(request)

        if request.url.path == '/auth' or request.method == 'OPTIONS':
            return await call_next(request)

        auth_header = request.headers.get('Authorization')
        if not auth_header or 'Bearer' not in auth_header:
            return JSONResponse(
                status_code=HTTP_401_UNAUTHORIZED,
                content={'detail': 'Authorization header missing'}
            )

        try:
            token = auth_header.split(' ')[1] if len(auth_header.split(' ')) > 1 else None
            if not token:
                return JSONResponse(
                    status_code=HTTP_401_UNAUTHORIZED,
                    content={'detail': 'Invalid token format'}
                )
            payload = TokenService.decode_token(token)
            session = await SessionService.get_active_session_by_client_id(
                client_id=payload['sub']
            )
            if not session:
                return JSONResponse(
                    status_code=HTTP_401_UNAUTHORIZED,
                    content={'detail': 'Invalid or inactive session'}
                )
            request.state.session = session

        except HTTPException as http_exc:
            return JSONResponse(
                status_code=http_exc.status_code,
                content={'detail': http_exc.detail}
            )
        except Exception as e:
            return JSONResponse(
                status_code=HTTP_401_UNAUTHORIZED,
                content={'detail': 'Authentication failed'}
            )
        return await call_next(request)



import os
import secrets
from typing import Optional

from fastapi import HTTPException
from kiss_ai_stack_types.enums import SessionScope
from starlette.status import HTTP_401_UNAUTHORIZED
from tortoise import Tortoise

from kiss_ai_stack_server.models.db.session import Session


class SessionService:
    @staticmethod
    def get_db_url() -> str:
        """
        Constructs the database URL for SQLite, defaulting to 'sessions.db' in the current directory.
        If a custom path is provided via the SESSION_DB_URL environment variable, it validates the path.
        """
        default_db_file = 'sessions.db'
        db_url = os.getenv('SESSION_DB_URL', f'sqlite://{default_db_file}')

        if db_url.startswith('sqlite://'):
            db_path = db_url[len('sqlite://'):]
            if not os.path.isabs(db_path):
                db_path = os.path.join(os.getcwd(), db_path)
                db_url = f'sqlite://{db_path}'
            db_dir = os.path.dirname(db_path)
            if db_dir and not os.path.exists(db_dir):
                os.makedirs(db_dir, exist_ok=True)

        return db_url

    @staticmethod
    async def connect():
        db_url = SessionService.get_db_url()
        await Tortoise.init(
            db_url=db_url,
            modules={'models': ['kiss_ai_stack_server.models.db']}
        )

    @staticmethod
    async def build_schema():
        await Tortoise.generate_schemas()

    @staticmethod
    async def get_session_by_client_id(client_id: str):
        return await Session.get_or_none(client_id=client_id)

    @staticmethod
    async def create_session(scope: SessionScope):
        return await Session.create(
            scope=scope
        )

    @staticmethod
    async def get_active_sessions():
        return await Session.filter(status=True)

    @staticmethod
    async def deactivate_session(client_id: str):
        session = await Session.get_or_none(client_id=client_id)
        if session:
            session.status = False
            await session.save()
            return True
        return False

    @staticmethod
    async def rotate_client_secret(client_id: str):
        session = await Session.get_or_none(client_id=client_id)
        if session:
            session.client_secret = secrets.token_urlsafe(32)
            await session.save()
            return session
        return None

    @staticmethod
    async def get_active_session_by_client_id(client_id: str) -> Session:
        """
        Validate session credentials and return the session.

        :param client_id: (str) The client ID to validate
        :returns Session: The validated active session
        :raises HTTPException: If session is invalid or inactive
        """
        session = await Session.get_or_none(
            client_id=client_id,
            status=True
        )

        if not session:
            raise HTTPException(
                status_code=HTTP_401_UNAUTHORIZED,
                detail='Invalid or inactive session'
            )
        return session

    @staticmethod
    async def create_or_get_session(
            scope: Optional[SessionScope] = None,
            client_id: Optional[str] = None,
            client_secret: Optional[str] = None
    ) -> Optional[Session]:
        if scope and not client_id and not client_secret:
            return await SessionService.create_session(scope)
        elif client_id and client_secret:
            session = await SessionService.get_active_session_by_client_id(
                client_id=client_id
            )
            if not session.client_secret == client_secret:
                raise HTTPException(
                    status_code=HTTP_401_UNAUTHORIZED,
                    detail='Invalid client_id or client_secret'
                )
            return session
        return None



from kiss_ai_stack_server.events.event_handlers import on_query
from kiss_ai_stack_server.models.db.session import Session
from kiss_ai_stack_server.services.kiss_ai_agent_service import KissAIAgentService
from kiss_ai_stack_types.models import QueryRequestBody, GenericResponseBody


@on_query
async def handle_query(data: QueryRequestBody, session: Session) -> GenericResponseBody:
    stack = KissAIAgentService()
    answer = await stack.generate_answer(
        agent_id=session.client_id,
        query=data.query
    )

    return GenericResponseBody(
        agent_id=session.client_id,
        result=answer.answer,
        extras={
            'query': data.query,
            'metadata': answer.metadata,
            'distances': answer.distances,
            'supporting_documents': answer.supporting_documents
        }
    )



import base64

from fastapi import HTTPException
from kiss_ai_stack.core.utilities.logger import LOG
from kiss_ai_stack_server.events.event_handlers import on_store
from kiss_ai_stack_server.models.db.session import Session
from kiss_ai_stack_server.services.kiss_ai_agent_service import KissAIAgentService
from kiss_ai_stack_server.utilities.temp_file_manager import temp_file_manager
from kiss_ai_stack_types.models import DocumentsRequestBody, GenericResponseBody


@on_store
async def handle_store(data: DocumentsRequestBody, session: Session = None) -> GenericResponseBody:
    stack = KissAIAgentService()
    temp_files = temp_file_manager()
    temp_dir = temp_files.create_temp_dir()
    files_stored = None

    try:
        file_paths = []

        for file_data in data.files:
            filename = file_data.name
            content = file_data.content
            safe_filename = temp_files.safe_file_path(temp_dir, filename)

            try:
                file_content = base64.b64decode(content)
            except Exception as e:
                raise HTTPException(
                    status_code=400,
                    detail=f'Invalid base64 content for file {filename}: {str(e)}'
                )
            with open(safe_filename, "wb") as buffer:
                buffer.write(file_content)
            file_paths.append(safe_filename)
        metadata = data.metadata or {}

        try:
            files_stored = await stack.store_data(
                agent_id=session.client_id,
                files=file_paths,
                metadata=metadata
            )
        except Exception as e:
            raise HTTPException(
                status_code=500,
                detail=f"Error storing documents: {str(e)}"
            )
    except Exception as e:
        LOG.error(f'ServerEvent :: on_store :: {str(e)}')
    finally:
        temp_files.cleanup_dir(temp_dir)
        return GenericResponseBody(
            agent_id=session.client_id,
            result='Files stored!',
            extras=files_stored
        )



from kiss_ai_stack_server.events.event_handlers import on_close
from kiss_ai_stack_server.models.db.session import Session
from kiss_ai_stack_server.services.kiss_ai_agent_service import KissAIAgentService
from kiss_ai_stack_server.services.session_service import SessionService
from kiss_ai_stack_types.enums import SessionScope
from kiss_ai_stack_types.models import QueryRequestBody, GenericResponseBody


@on_close
async def handle_close(data: QueryRequestBody, session: Session = None) -> GenericResponseBody:
    stack = KissAIAgentService()
    cleanup = session.scope == SessionScope.TEMPORARY
    await stack.destroy_agent(agent_id=session.client_id, cleanup=cleanup)
    await SessionService.deactivate_session(session.client_id)

    return GenericResponseBody(
        agent_id=session.client_id,
        result='Good bye!'
    )



from setuptools import setup, find_packages

setup(
    name="kiss-ai-stack-server",
    version="0.1.0-alpha15",
    description="KISS AI Stack's Server stub - Simplify AI Agent Development",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author="KISS AI Stack, Lahiru Pathirage",
    license="MIT",
    python_requires='>=3.12',
    packages=find_packages(where='src'),
    package_dir={'': 'src'},
    install_requires=[
        "fastapi~=0.115.5",
        "uvicorn~=0.32.1",
        "websockets~=14.1",
        "jose~=1.0.0",
        "bcrypt~=4.2.1",
        "tortoise-orm~=0.22.2",
        "PyJWT~=2.10.1",
        "starlette~=0.41.3",
        "kiss-ai-stack-types~=0.1.0a2",
        "kiss_ai_stack_core~=0.1.0a10"
    ],
    keywords=["ai", "agent", "machine-learning", "llm", "document-processing"],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.12",
        "Topic :: Scientific/Engineering :: Artificial Intelligence"
    ],
    project_urls={
        "Homepage": "https://github.com/kiss-ai-stack",
        "Repository": "https://github.com/kiss-ai-stack",
        "Documentation": "https://github.com/kiss-ai-stack/kiss-ai-stack-core/main/README.md"
    }
)
